﻿
using ClosedXML.Excel;
using DocumentFormat.OpenXml.Spreadsheet;
using System;
using System.Data;
using System.Data.OleDb;
using System.IO;

namespace MilitaryGuaranteeCard
{
    /// <summary>
    /// Excel帮助类
    /// </summary>
    public class ExcelHelper
    {
        private static object m_lock = new object();

        private static ExcelHelper m_excelHelper = null;

        public static ExcelHelper GetInstance
        {
            get
            {
                if (m_excelHelper == null)
                {
                    lock (m_lock)
                    {
                        if (m_excelHelper == null)
                        {
                            m_excelHelper = new ExcelHelper();
                        }
                        return m_excelHelper;
                    }
                }
                return m_excelHelper;
            }
        }

        public static DataSet ReadExcel(string path)
        {
            try
            {
                string fileSuffix = System.IO.Path.GetExtension(path);
                if (string.IsNullOrEmpty(fileSuffix))
                    return null;
                using (DataSet ds = new DataSet())
                {
                    string connStr = "";
                    if (fileSuffix == ".xls")
                        connStr = "Provider=Microsoft.Jet.OLEDB.4.0;Data Source=" + path + ";Extended Properties=\"Excel 8.0;HDR = Yes;IMEX=1\"";
                    else
                        connStr = "Provider=Microsoft.ACE.OLEDB.12.0;Data Source=" + path + ";Extended Properties=\"Excel 12.0;HDR = Yes;IMEX=1\"";
                    string sql_select = "SELECT * FROM [SQL Results$]";
                    using (OleDbConnection conn = new OleDbConnection(connStr))
                    using (OleDbDataAdapter cmd = new OleDbDataAdapter(sql_select, conn))
                    {
                        OpenConn(conn);
                        cmd.Fill(ds);
                        CloseConn(conn);
                    }
                    if (ds == null || ds.Tables.Count <= 0) return null;
                    //Clear.BHC.Common.logManager.WriteLogByPath("Excel_Log", "Excel表数据查询成功");
                    return ds;
                }
            }
            catch (Exception ex)
            {
                throw new Exception($"ReadExcel函数发生异常:{ex.Message}");
            }
        }
        public static void OpenConn(OleDbConnection conn)
        {
            conn.Open();
        }

        public static void CloseConn(OleDbConnection conn)
        {
            conn.Close();
        }
        /// <summary>
        /// 根据条件获取表格
        /// </summary>
        /// <param name="path">路径</param>
        /// <param name="sheetname">表名</param>
        /// <param name="condition">条件</param>
        /// <returns></returns>
        /// <exception cref="Exception"></exception>
        public static DataSet ReadExcel2(string path, string sheetName, string condition)
        {
            if (string.IsNullOrEmpty(path) || string.IsNullOrEmpty(sheetName))
            {
                throw new ArgumentException("路径或工作表名称不能为空。");
            }

            try
            {
                // 判断文件后缀，选择合适的连接字符串
                string fileSuffix = Path.GetExtension(path);
                if (string.IsNullOrEmpty(fileSuffix))
                {
                    throw new FileNotFoundException("文件后缀无法识别，请检查路径是否正确。");
                }

                string connStr = "";
                if (fileSuffix.ToLower() == ".xls")
                {
                    connStr = $"Provider=Microsoft.Jet.OLEDB.4.0;Data Source={path};Extended Properties=\"Excel 8.0;HDR=Yes;IMEX=1\"";
                }
                else if (fileSuffix.ToLower() == ".xlsx")
                {
                    connStr = $"Provider=Microsoft.ACE.OLEDB.12.0;Data Source={path};Extended Properties=\"Excel 12.0;HDR=Yes;IMEX=1\"";
                }
                else
                {
                    throw new NotSupportedException("仅支持 .xls 和 .xlsx 格式的文件。");
                }

                string query = $"SELECT * FROM [{sheetName}$]";
                if (!string.IsNullOrEmpty(condition))
                {
                    query += $" {condition}";
                }

                using (OleDbConnection conn = new OleDbConnection(connStr))
                {
                    conn.Open();
                    using (OleDbCommand cmd = new OleDbCommand(query, conn))
                    using (OleDbDataAdapter adapter = new OleDbDataAdapter(cmd))
                    {
                        DataSet ds = new DataSet();
                        adapter.Fill(ds);
                        if (ds.Tables.Count > 0)
                        {
                            return ds;
                        }
                        return null;
                    }
                }
            }
            catch (OleDbException oleDbEx)
            {
                throw new Exception($"数据库操作异常：{oleDbEx.Message}");
            }
            catch (Exception ex)
            {
                throw new Exception($"ReadExcel2函数发生异常:{ex}");
            }
        }


        /// <summary>
        /// 使用外部控件读取excel数据
        /// </summary>
        /// <param name="path"></param>
        /// <param name="sheetname"></param>
        /// <returns></returns>
        public static IXLWorksheet ReadExcel3(string path, string sheetname)
        {
            try
            {
                using (var workbook = new XLWorkbook(path))
                {
                    var worksheet = workbook.Worksheet(sheetname);

                    return worksheet;
                }
            }
            catch (Exception ex)
            {
                throw new Exception($"ReadExcel3函数发生异常:{ex.Message}");
            }
        }
    }
}
