﻿using Newtonsoft.Json;
using System;
using System.Data;
using System.Dynamic;
using System.IO;
using System.Runtime.ExceptionServices;
using System.Runtime.InteropServices;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;

namespace MilitaryGuaranteeCard
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            btnPowerOn.Enabled = false;
            btnPowerOff.Enabled = false;
            btnReadCard.Enabled = false;
        }
        /// <summary>
        /// 打开标识
        /// </summary>
        private static bool _openHandle=false;

        /// <summary>
        /// 打开/关闭设备
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        [HandleProcessCorruptedStateExceptions]
        private void btnOpen_Click(object sender, EventArgs e)
        {
            try
            {
                // 接收设备端口,并转换为全大写
                String devPort = txtDevPort.Text.Trim().ToUpper();
                if (!_openHandle)
                {
                    // 判断设备端口是否为null
                    if (!String.IsNullOrEmpty(devPort))
                    {
                        // 调用Dll,打开设备通讯
                        if (DllImport.OpenComm(devPort).Equals(0))
                        {
                            _openHandle = true;
                            btnPowerOn.Enabled=true;
                            MessageBox.Show("设备通讯打开成功", "提示", MessageBoxButtons.OK, MessageBoxIcon.Information);
                            btnOpen.Text = "关闭设备";
                            btnOpen.Enabled = true;
                        }
                        else
                        {
                            MessageBox.Show("设备通讯打开失败", "警告", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                        }
                    }
                    else
                    {
                        MessageBox.Show("请输入设备端口", "警告", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    }
                }
                else
                {
                    // 调用Dll,关闭设备通讯
                    if (DllImport.CloseComm().Equals(0))
                    {
                        _openHandle = false;
                        MessageBox.Show("设备通讯关闭成功", "提示", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        btnOpen.Text = "打开设备";
                    }
                    else
                    {
                        MessageBox.Show("设备通讯关闭失败", "警告", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    }
                    DllImport.UnloadTargetDll();
                }
            }
            catch (Exception ex)
            {
                Logger.Instance.ErrorLogs($"设备通讯操作函数发生异常", ex);
            }
            
        }
        /// <summary>
        /// IC卡片上电
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        [HandleProcessCorruptedStateExceptions]
        private void btnPowerOn_Click(object sender, EventArgs e)
        {
            try
            {
                // 判断打开标识
                if (_openHandle)
                {
                    // 定义缓冲区大小
                    const int bufferSize = 256;
                    // 托管缓冲区
                    byte[] managedBuffer = new byte[bufferSize];
                    // 初始长度
                    uint dataLength = (uint)managedBuffer.Length; 
                    // 分配非托管内存
                    IntPtr recData = Marshal.AllocHGlobal(bufferSize);
                    // 初始化非托管内存
                    Marshal.Copy(managedBuffer, 0, recData, bufferSize);
                    // 调用Dll,IC卡片上电
                    if (DllImport.PowerOn(recData, ref dataLength).Equals(0))
                    {
                        MessageBox.Show("IC卡片上电成功", "提示", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        btnPowerOn.Enabled = false;
                        btnPowerOff.Enabled = true;
                        btnReadCard.Enabled = true;
                    }
                    else
                    {
                        MessageBox.Show("IC卡片上电失败", "警告", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    }
                }
                else
                {
                    MessageBox.Show("设备通讯未打开", "警告", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }
            catch (Exception ex)
            {
                Logger.Instance.ErrorLogs($"IC卡片上电函数发生异常", ex);
            }
        }
        /// <summary>
        /// IC卡片下电
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        [HandleProcessCorruptedStateExceptions]
        private void btnPowerOff_Click(object sender, EventArgs e)
        {
            try
            {
                // 判断打开标识
                if (_openHandle)
                {
                    // 定义缓冲区大小
                    const int bufferSize = 256;
                    // 托管缓冲区
                    byte[] managedBuffer = new byte[bufferSize];
                    // 初始长度
                    uint dataLength = (uint)managedBuffer.Length;
                    // 分配非托管内存
                    IntPtr recData = Marshal.AllocHGlobal(bufferSize);
                    // 初始化非托管内存
                    Marshal.Copy(managedBuffer, 0, recData, bufferSize);
                    if (DllImport.PowerOff(recData, ref dataLength).Equals(0))
                    {
                        MessageBox.Show("IC卡片下电成功", "提示", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        rtbOutput.Text = null;
                        btnPowerOn.Enabled = true;
                        btnPowerOff.Enabled = false;
                        btnReadCard.Enabled = false;
                    }
                    else
                    {
                        MessageBox.Show("IC卡片下电失败", "警告", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    }
                }
                else
                {
                    MessageBox.Show("设备通讯未打开", "警告", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }
            catch (Exception ex)
            {
                Logger.Instance.ErrorLogs($"IC卡片下电函数发生异常", ex);
            }
           
        }
        /// <summary>
        /// 读取卡片
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        [HandleProcessCorruptedStateExceptions]
        private void btnReadCard_Click(object sender, EventArgs e)
        {
            try
            {
                // 判断打开标识
                if (_openHandle)
                {
                    // 定义缓冲区大小
                    const int bufferSize = 256;
                    // 托管缓冲区
                    byte[] managedBuffer = new byte[bufferSize];
                    // 初始长度
                    uint dataLength = (uint)managedBuffer.Length;
                    // 分配非托管内存
                    IntPtr recData = Marshal.AllocHGlobal(bufferSize);
                    // 初始化非托管内存
                    Marshal.Copy(managedBuffer, 0, recData, bufferSize);
                    if (!DllImport.Authenfic1608(recData, ref dataLength).Equals(0))
                    {
                        MessageBox.Show("双向认证未通过", "警告", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                        return;
                    }
                    int keyNo = ((8 * 0) + 0) + 1;
                    // 初始长度
                    dataLength = (uint)managedBuffer.Length;
                    // 分配非托管内存
                    recData = Marshal.AllocHGlobal(bufferSize);
                    // 初始化非托管内存
                    Marshal.Copy(managedBuffer, 0, recData, bufferSize);
                    if (!DllImport.Chk1608Secret(keyNo, recData, ref dataLength).Equals(0))
                    {
                        MessageBox.Show("密码核对未通过", "警告", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                        return;
                    }
                    #region 读取账号类型
                    string filePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory,"PLA", $"{txtExcelName.Text.Trim()}.xls");

                    string cardType = "AT88SC1608";
                    string accountType = "0";  // 假设这是一个数字，不需要引号
                    string itemName = "ACCOUNT_TYPE".ToUpper();

                    // 确保条件拼接时，字符串类型字段加引号，数字类型字段保持不变
                    string sqlCondition = $"WHERE ICCARD_TYPE = '{cardType}' AND ACCOUNT_TYPE = {accountType} AND ITEM_ID = '{itemName}'";
                    // 查询第一行数据
                    DataRow findRow = ExcelHelper.ReadExcel2(filePath, txtSheetName.Text.Trim(), sqlCondition)?.Tables[0]?.Rows[0];
                    if (findRow == null)
                    {
                        MessageBox.Show("查询数据为null", "警告", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                        return;
                    }
                    int section_no = Convert.ToInt32(findRow["SECTION_NO"]);
                    int start_pos = Convert.ToInt32(findRow["START_POS"]);
                    int item_len = Convert.ToInt32(findRow["ITEM_LEN"]);
                    int numeric_indicator = Convert.ToInt32(findRow["NUMERIC_INDICATOR"]);
                    int fee_indicator = Convert.ToInt32(findRow["FEE_INDICATOR"]);
                    string item_name = findRow["ITEM_NAME"].ToString();
                    #endregion
                    #region 读卡操作
                    itemName = "ACCOUNT_INFO_SECTION".ToUpper();

                    // 确保条件拼接时，字符串类型字段加引号，数字类型字段保持不变
                    sqlCondition = $"WHERE ICCARD_TYPE = '{cardType}' AND ACCOUNT_TYPE = {accountType} AND ITEM_ID = '{itemName}'";
                    // 查询第一行数据
                    findRow = ExcelHelper.ReadExcel2(filePath, txtSheetName.Text.Trim(), sqlCondition)?.Tables[0]?.Rows[0];
                    if (findRow == null)
                    {
                        MessageBox.Show("查询数据为null", "警告", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                        return;
                    }
                    section_no = Convert.ToInt32(findRow["SECTION_NO"]);
                    start_pos = Convert.ToInt32(findRow["START_POS"]);
                    item_len = Convert.ToInt32(findRow["ITEM_LEN"]);
                    numeric_indicator = Convert.ToInt32(findRow["NUMERIC_INDICATOR"]);
                    fee_indicator = Convert.ToInt32(findRow["FEE_INDICATOR"]);
                    item_name = findRow["ITEM_NAME"].ToString();

                    // 初始长度
                    dataLength = (uint)managedBuffer.Length;
                    // 分配非托管内存
                    recData = Marshal.AllocHGlobal(bufferSize);
                    // 初始化非托管内存
                    Marshal.Copy(managedBuffer, 0, recData, bufferSize);
                    dynamic result = new ExpandoObject();
                    if (!DllImport.ReadCard((uint)section_no, (uint)start_pos, (uint)item_len, recData, ref dataLength).Equals(0))
                    {
                        MessageBox.Show("读取卡片失败", "警告", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                        return;
                    }
                    else
                    {
                        // 从非托管内存读取数据到托管缓冲区
                        byte[] dataBytes = new byte[dataLength];
                        Marshal.Copy(recData, dataBytes, 0, (int)dataLength);
                        result.InsuranceNo = ReadItem(0, "INSURANCE_NO", dataBytes);
                        result.CardNo = ReadItem(0, "CARD_NO", dataBytes);
                        result.PatientId = ReadItem(0, "PATIENT_ID", dataBytes);
                        result.Name = ReadItem(0, "NAME", dataBytes);
                        result.Sex= ReadItem(0,"SEX", dataBytes);
                        result.Birthday = ReadItem(0, "DATE_OF_BIRTH", dataBytes);
                        // 解析出生日期字符串并转换为日期时间类型
                        //result.Birthday = $"{birth.Substring(0, 4)}-{birth.Substring(4, 2)}-{birth.Substring(6, 2)}";
                        result.BirthPlace = ReadItem(0, "BIRTH_PLACE", dataBytes);
                        result.Nation = ReadItem(0, "NATION", dataBytes);
                        result.MaritalStatus = ReadItem(0, "MARITAL_STATUS", dataBytes);
                        result.IdentityClass = ReadItem(0, "IDENTITY_CLASS", dataBytes);
                        // 行政职位
                        result.GradeOfDuty = ReadItem(0, "GRADE_OF_DUTY", dataBytes);
                        // 技术级别
                        result.GradeOfTechnic = ReadItem(0, "GRADE_OF_TECHNIC", dataBytes);
                        // 技术职务
                        result.GradeOfTitle = ReadItem(0, "GRADE_OF_TITLE", dataBytes);
                        result.WorkingStatus = ReadItem(0, "WORKING_STATUS", dataBytes);
                        // 人员基本标准经费
                        result.FundType = ReadItem(0, "FUND_TYPE", dataBytes);
                        // 人员补助标准经费
                        result.SubsidyType = ReadItem(0, "SUBSIDY_TYPE", dataBytes);

                        result.InsuranceType = ReadItem(0, "INSURANCE_TYPE", dataBytes);
                        result.Unit = ReadItem(0, "unit", dataBytes);
                        result.DesignatedHospital = ReadItem(0, "DESIGNATED_HOSPITAL", dataBytes);
                        result.DesignatedPoliclinic = ReadItem(0, "DESIGNATED_POLICLINIC", dataBytes);

                        result.RelatedInsuranceNo = ReadItem(0, "RELATED_INSURANCE_NO_1", dataBytes);
                        result.Relationship = ReadItem(0, "RELATIONSHIP_1", dataBytes);
                        result.ExpirationDate = ReadItem(0, "EXPIRATION_DATE", dataBytes);
                        // 解析日期字符串并转换为日期时间类型
                        //result.ExpirationDate = $"{expiration.Substring(0, 4)}-{expiration.Substring(4, 2)}-{expiration.Substring(6, 2)}";
                        result.AccountRatedAmount = ReadItem(0, "ACCOUNT_RATED_AMOUNT", dataBytes);
                        result.AccountInitialAmount = ReadItem(0, "ACCOUNT_INITIAL_AMOUNT", dataBytes);
                        result.AccountBalance = ReadItem(0, "ACCOUNT_BALANCE", dataBytes);
                        result.SettleDate = ReadItem(0, "SETTLE_DATE", dataBytes);
                        // 解析日期字符串并转换为日期时间类型
                        //result.SettleDate = $"{settle.Substring(0, 4)}-{settle.Substring(4, 2)}-{settle.Substring(6, 2)}";
                        result.AssignCenter = ReadItem(0, "ASSIGN_CENTER", dataBytes);
                        result.AssignDate = ReadItem(0, "ASSIGN_DATE", dataBytes);
                        // 解析日期字符串并转换为日期时间类型
                        //result.AssignDate = $"{assign.Substring(0, 4)}-{assign.Substring(4, 2)}-{assign.Substring(6, 2)}";
                    }
                    rtbOutput.Text=JsonConvert.SerializeObject(result);
                    #endregion
                }
                else
                {
                    MessageBox.Show("设备通讯未打开", "警告", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }
            catch (Exception ex)
            {
                Logger.Instance.ErrorLogs($"读取卡片函数发生异常", ex);
            }
        }
        /// <summary>
        /// 读取节点字段
        /// </summary>
        /// <param name="accountType">账户类型</param>
        /// <param name="itemName">字段</param>
        /// <param name="strData">数据</param>
        /// <returns></returns>
        private String ReadItem(int accountType,string itemName, byte[] strData)
        {
            String itemValue = String.Empty;
            try
            {
                string filePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory,"PLA", $"{txtExcelName.Text.Trim()}.xls");
                string cardType = "AT88SC1608";
                // 确保条件拼接时，字符串类型字段加引号，数字类型字段保持不变
                string sqlCondition = $"WHERE ICCARD_TYPE = '{cardType}' AND ACCOUNT_TYPE = {accountType.ToString()} AND ITEM_ID = '{itemName.ToUpper()}'";
                // 查询第一行数据
                DataRow findRow = ExcelHelper.ReadExcel2(filePath, txtSheetName.Text.Trim(), sqlCondition)?.Tables[0]?.Rows[0];
                if (findRow == null)
                {
                    MessageBox.Show("查询数据为null", "警告", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    return itemValue;
                }
                int section_no = Convert.ToInt32(findRow["SECTION_NO"]);
                int start_pos = Convert.ToInt32(findRow["START_POS"]);
                int item_len = Convert.ToInt32(findRow["ITEM_LEN"]);
                int numeric_indicator = Convert.ToInt32(findRow["NUMERIC_INDICATOR"]);
                int fee_indicator = Convert.ToInt32(findRow["FEE_INDICATOR"]);
                string item_name = findRow["ITEM_NAME"].ToString();
                // 创建一个新的字节数组来存放截取的数据
                byte[] subArray = new byte[item_len];
                // 使用 Array.Copy 截取子数组
                Array.Copy(strData, start_pos, subArray, 0, item_len);
                byte[] ls_ttc = new byte[4];
                if (numeric_indicator.Equals(1))
                {
                    for (int i = 0; i < ls_ttc.Length; i++)
                    {
                        if (i < item_len)
                        {
                            ls_ttc[i] = subArray[i];
                            continue;
                        }
                    }
                    ulong temp = 0;
                    if (DllImport.HexByteToLong(ls_ttc, ref temp) < 0)
                    {
                        itemValue = string.Empty;
                    }
                    if (fee_indicator.Equals(1))
                    {
                        decimal ldItem = temp / 100m; // 将长整型转换为Decimal并除以100
                        itemValue = ldItem.ToString("F2"); // 格式化为两位小数的字符串
                    }
                    else
                    {
                        itemValue = temp.ToString();
                    }

                }
                else
                {
                    itemValue = Encoding.GetEncoding("GB2312").GetString(subArray).TrimEnd('\0').Trim();
                }
            }
            catch (Exception ex)
            {
                Logger.Instance.ErrorLogs($"ReadItem函数发生异常", ex);
            }
            return itemValue;
        }
    }
}
