﻿using System;
using System.IO;
using System.Reflection;
using log4net;
using log4net.Config;
using log4net.Repository;

namespace MilitaryGuaranteeCard
{
    /// <summary>
    /// 日志
    /// </summary>
    public class Logger
    {
        #region LazyLoading

        private static readonly Lazy<Logger> Lazy = new Lazy<Logger>(() => new Logger());
        public static Logger Instance => Lazy.Value;

        #endregion

        #region Define

        /// <summary>
        /// 将消息记录到 log4net 框架中
        /// </summary>
        private ILog _logger;

        /// <summary>
        /// 存储库
        /// </summary>
        private readonly ILoggerRepository _repository;
        #endregion
        /// <summary>
        /// 构造器
        /// </summary>
        public Logger()
        {
            // 日志目录
            String logDirectory = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Logs");
            // 手动创建各个子文件夹
            string[] subDirectories = { "NormalLogs", "ServiceLogs", "WarnLogs", "ErrorLogs" };
            foreach (var subDir in subDirectories)
            {
                string fullPath = Path.Combine(logDirectory, subDir);
                if (!Directory.Exists(fullPath))
                {
                    Directory.CreateDirectory(fullPath);
                }
            }
            // 获取当前应用程序的log4net日志存储库
            _repository = LogManager.CreateRepository("PLA");
            // 使用指定的配置文件配置
            XmlConfigurator.Configure(_repository, new FileInfo(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "log4net.config")));
        }

        #region Methods

        /// <summary>
        /// 普通日志
        /// </summary>
        /// <param name="message">消息</param>
        /// <param name="exception">执行过程中发生的错误</param>
        public void NormalLogs(String message, Exception exception)
        {
            //检索或创建命名记录器
            _logger = LogManager.GetLogger(_repository.Name, "NormalLogs");
            if (exception != null)
            {
                _logger.Info(message, exception);
            }
            else
            {
                _logger.Info(message);
            }
        }
        /// <summary>
        /// 服务日志
        /// </summary>
        /// <param name="message">消息</param>
        /// <param name="exception">执行过程中发生的错误</param>
        public void ServiceLogs(String message, Exception exception)
        {
            //检索或创建命名记录器
            _logger = LogManager.GetLogger(_repository.Name, "ServiceLogs");
            if (exception != null)
            {
                _logger.Info(message, exception);
            }
            else
            {
                _logger.Info(message);
            }
        }
        /// <summary>
        /// 警告日志
        /// </summary>
        /// <param name="message"></param>
        /// <param name="exception"></param>
        public void WarnLogs(String message, Exception exception)
        {
            // 检索或创建命名记录器
            _logger = LogManager.GetLogger(_repository.Name, "WarnLogs");
            if (exception!=null)
            {
                _logger.Warn(message,exception);
            }
            else
            {
                _logger.Warn(message);
            } 
        }
        /// <summary>
        /// 错误日志
        /// </summary>
        /// <param name="message">消息</param>
        /// <param name="exception">执行过程中发生的错误</param>
        public void ErrorLogs(String message, Exception exception)
        {
            //检索或创建命名记录器
            _logger = LogManager.GetLogger(_repository.Name, "ErrorLogs");
            if (exception != null)
            {
                _logger.Error(message, exception);
            }
            else
            {
                _logger.Error(message);
            }
        }

        #endregion
    }
}